@@ -47,6 +47,19 @@ class AskSymmetricHandler(CommonHandler):
                 return True
             return ask(Q.symmetric(MatMul(*mmul.args[1:-1])), assumptions)
 
+    @staticmethod
+    def MatPow(expr, assumptions):
+        # only for integer powers
+        base, exp = expr.args
+        int_exp = ask(Q.integer(exp), assumptions)
+        if not int_exp:
+            return None
+        non_negative = ask(~Q.negative(exp), assumptions)
+        if (non_negative or non_negative == False
+                            and ask(Q.invertible(base), assumptions)):
+            return ask(Q.symmetric(base), assumptions)
+        return None
+
     @staticmethod
     def MatAdd(expr, assumptions):
         return all(ask(Q.symmetric(arg), assumptions) for arg in expr.args)
@@ -100,6 +113,17 @@ class AskInvertibleHandler(CommonHandler):
                for arg in mmul.args):
             return False
 
+    @staticmethod
+    def MatPow(expr, assumptions):
+        # only for integer powers
+        base, exp = expr.args
+        int_exp = ask(Q.integer(exp), assumptions)
+        if not int_exp:
+            return None
+        if exp.is_negative == False:
+            return ask(Q.invertible(base), assumptions)
+        return None
+
     @staticmethod
     def MatAdd(expr, assumptions):
         return None
@@ -142,6 +166,15 @@ class AskOrthogonalHandler(CommonHandler):
                 for arg in mmul.args):
             return False
 
+    @staticmethod
+    def MatPow(expr, assumptions):
+        # only for integer powers
+        base, exp = expr.args
+        int_exp = ask(Q.integer(exp), assumptions)
+        if int_exp:
+            return ask(Q.orthogonal(base), assumptions)
+        return None
+
     @staticmethod
     def MatAdd(expr, assumptions):
         if (len(expr.args) == 1 and
@@ -150,7 +183,8 @@ class AskOrthogonalHandler(CommonHandler):
 
     @staticmethod
     def MatrixSymbol(expr, assumptions):
-        if not expr.is_square:
+        if (not expr.is_square or
+                        ask(Q.invertible(expr), assumptions) is False):
             return False
         if Q.orthogonal(expr) in conjuncts(assumptions):
             return True
@@ -190,9 +224,19 @@ class AskUnitaryHandler(CommonHandler):
                 for arg in mmul.args):
             return False
 
+    @staticmethod
+    def MatPow(expr, assumptions):
+        # only for integer powers
+        base, exp = expr.args
+        int_exp = ask(Q.integer(exp), assumptions)
+        if int_exp:
+            return ask(Q.unitary(base), assumptions)
+        return None
+
     @staticmethod
     def MatrixSymbol(expr, assumptions):
-        if not expr.is_square:
+        if (not expr.is_square or
+                        ask(Q.invertible(expr), assumptions) is False):
             return False
         if Q.unitary(expr) in conjuncts(assumptions):
             return True
@@ -230,6 +274,15 @@ class AskFullRankHandler(CommonHandler):
         if all(ask(Q.fullrank(arg), assumptions) for arg in expr.args):
             return True
 
+    @staticmethod
+    def MatPow(expr, assumptions):
+        # only for integer powers
+        base, exp = expr.args
+        int_exp = ask(Q.integer(exp), assumptions)
+        if int_exp and ask(~Q.negative(exp), assumptions):
+            return ask(Q.fullrank(base), assumptions)
+        return None
+
     Identity = staticmethod(CommonHandler.AlwaysTrue)
 
     ZeroMatrix = staticmethod(CommonHandler.AlwaysFalse)
@@ -262,6 +315,12 @@ class AskPositiveDefiniteHandler(CommonHandler):
             return ask(Q.positive_definite(
                 MatMul(*mmul.args[1:-1])), assumptions)
 
+    @staticmethod
+    def MatPow(expr, assumptions):
+        # a power of a positive definite matrix is positive definite
+        if ask(Q.positive_definite(expr.args[0]), assumptions):
+            return True
+
     @staticmethod
     def MatAdd(expr, assumptions):
         if all(ask(Q.positive_definite(arg), assumptions)
@@ -308,6 +367,19 @@ class AskUpperTriangularHandler(CommonHandler):
         if all(ask(Q.upper_triangular(arg), assumptions) for arg in expr.args):
             return True
 
+    @staticmethod
+    def MatPow(expr, assumptions):
+        # only for integer powers
+        base, exp = expr.args
+        int_exp = ask(Q.integer(exp), assumptions)
+        if not int_exp:
+            return None
+        non_negative = ask(~Q.negative(exp), assumptions)
+        if (non_negative or non_negative == False
+                            and ask(Q.invertible(base), assumptions)):
+            return ask(Q.upper_triangular(base), assumptions)
+        return None
+
     @staticmethod
     def MatrixSymbol(expr, assumptions):
         if Q.upper_triangular(expr) in conjuncts(assumptions):
@@ -348,6 +420,19 @@ class AskLowerTriangularHandler(CommonHandler):
         if all(ask(Q.lower_triangular(arg), assumptions) for arg in expr.args):
             return True
 
+    @staticmethod
+    def MatPow(expr, assumptions):
+        # only for integer powers
+        base, exp = expr.args
+        int_exp = ask(Q.integer(exp), assumptions)
+        if not int_exp:
+            return None
+        non_negative = ask(~Q.negative(exp), assumptions)
+        if (non_negative or non_negative == False
+                            and ask(Q.invertible(base), assumptions)):
+            return ask(Q.lower_triangular(base), assumptions)
+        return None
+
     @staticmethod
     def MatrixSymbol(expr, assumptions):
         if Q.lower_triangular(expr) in conjuncts(assumptions):
@@ -389,6 +474,19 @@ class AskDiagonalHandler(CommonHandler):
         if all(ask(Q.diagonal(m), assumptions) for m in matrices):
             return True
 
+    @staticmethod
+    def MatPow(expr, assumptions):
+        # only for integer powers
+        base, exp = expr.args
+        int_exp = ask(Q.integer(exp), assumptions)
+        if not int_exp:
+            return None
+        non_negative = ask(~Q.negative(exp), assumptions)
+        if (non_negative or non_negative == False
+                            and ask(Q.invertible(base), assumptions)):
+            return ask(Q.diagonal(base), assumptions)
+        return None
+
     @staticmethod
     def MatAdd(expr, assumptions):
         if all(ask(Q.diagonal(arg), assumptions) for arg in expr.args):
@@ -424,6 +522,14 @@ class AskDiagonalHandler(CommonHandler):
     def DiagonalMatrix(expr, assumptions):
         return True
 
+    @staticmethod
+    def DiagMatrix(expr, assumptions):
+        return True
+
+    @staticmethod
+    def Identity(expr, assumptions):
+        return True
+
     Factorization = staticmethod(partial(_Factorization, Q.diagonal))
 
 
@@ -447,6 +553,17 @@ class AskIntegerElementsHandler(CommonHandler):
     def MatAdd(expr, assumptions):
         return test_closed_group(expr, assumptions, Q.integer_elements)
 
+    @staticmethod
+    def MatPow(expr, assumptions):
+        # only for integer powers
+        base, exp = expr.args
+        int_exp = ask(Q.integer(exp), assumptions)
+        if not int_exp:
+            return None
+        if exp.is_negative == False:
+            return ask(Q.integer_elements(base), assumptions)
+        return None
+
     HadamardProduct, Determinant, Trace, Transpose = [MatAdd]*4
 
     ZeroMatrix, Identity = [staticmethod(CommonHandler.AlwaysTrue)]*2
@@ -461,8 +578,21 @@ class AskRealElementsHandler(CommonHandler):
     def MatAdd(expr, assumptions):
         return test_closed_group(expr, assumptions, Q.real_elements)
 
-    HadamardProduct, Determinant, Trace, Transpose, Inverse, \
-            Factorization = [MatAdd]*6
+    @staticmethod
+    def MatPow(expr, assumptions):
+        # only for integer powers
+        base, exp = expr.args
+        int_exp = ask(Q.integer(exp), assumptions)
+        if not int_exp:
+            return None
+        non_negative = ask(~Q.negative(exp), assumptions)
+        if (non_negative or non_negative == False
+                            and ask(Q.invertible(base), assumptions)):
+            return ask(Q.real_elements(base), assumptions)
+        return None
+
+    HadamardProduct, Determinant, Trace, Transpose, \
+            Factorization = [MatAdd]*5
 
     MatMul = staticmethod(partial(MatMul_elements, Q.real_elements, Q.real))
     MatrixSlice = staticmethod(partial(MS_elements, Q.real_elements))
@@ -474,6 +604,19 @@ class AskComplexElementsHandler(CommonHandler):
     def MatAdd(expr, assumptions):
         return test_closed_group(expr, assumptions, Q.complex_elements)
 
+    @staticmethod
+    def MatPow(expr, assumptions):
+        # only for integer powers
+        base, exp = expr.args
+        int_exp = ask(Q.integer(exp), assumptions)
+        if not int_exp:
+            return None
+        non_negative = ask(~Q.negative(exp), assumptions)
+        if (non_negative or non_negative == False
+                            and ask(Q.invertible(base), assumptions)):
+            return ask(Q.complex_elements(base), assumptions)
+        return None
+
     HadamardProduct, Determinant, Trace, Transpose, Inverse, \
          Factorization = [MatAdd]*6
 
