@@ -49,7 +49,7 @@ def dummy_sgs(dummies, sym, n):
     ========
 
     >>> from sympy.combinatorics.tensor_can import dummy_sgs
-    >>> dummy_sgs(range(2, 8), 0, 8)
+    >>> dummy_sgs(list(range(2, 8)), 0, 8)
     [[0, 1, 3, 2, 4, 5, 6, 7, 8, 9], [0, 1, 2, 3, 5, 4, 6, 7, 8, 9],
      [0, 1, 2, 3, 4, 5, 7, 6, 8, 9], [0, 1, 4, 5, 2, 3, 6, 7, 8, 9],
      [0, 1, 2, 3, 6, 7, 4, 5, 8, 9]]
@@ -168,6 +168,9 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
     """
     Butler-Portugal algorithm for tensor canonicalization with dummy indices
 
+    Parameters
+    ==========
+
       dummies
         list of lists of dummy indices,
         one list for each type of index;
@@ -194,9 +197,14 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
       g
         permutation representing the tensor.
 
+    Returns
+    =======
+
     Return 0 if the tensor is zero, else return the array form of
     the permutation representing the canonical form of the tensor.
 
+    Notes
+    =====
 
     A tensor with dummy indices can be represented in a number
     of equivalent ways which typically grows exponentially with
@@ -222,7 +230,7 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
     (the indices including both the contravariant and the covariant ones)
     can be written as
 
-    `t = T(ind[g[0],..., ind[g[n-1]])`,
+    `t = T(ind[g[0]],..., ind[g[n-1]])`,
 
     where `n= len(ind)`;
     `g` has size `n + 2`, the last two indices for the sign of the tensor
@@ -238,7 +246,8 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
     these symmetries, one can represent the tensor by `g`, which transforms
     as
 
-    `g -> d*g*s`, so it belongs to the coset `D*g*S`.
+    `g -> d*g*s`, so it belongs to the coset `D*g*S`, or in other words
+    to the set of all permutations allowed by the slot and dummy symmetries.
 
     Let us explain the conventions by an example.
 
@@ -249,14 +258,14 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
 
     and symmetric metric, find the tensor equivalent to it which
     is the lowest under the ordering of indices:
-    lexicographic ordering `d1, d2, d3` then and contravariant index
+    lexicographic ordering `d1, d2, d3` and then contravariant
     before covariant index; that is the canonical form of the tensor.
 
     The canonical form is `-T^{d1 d2 d3}{}_{d1 d2 d3}`
     obtained using `T^{a0 a1 a2 a3 a4 a5} = -T^{a2 a1 a0 a3 a4 a5}`.
 
     To convert this problem in the input for this function,
-    use the following labelling of the index names
+    use the following ordering of the index names
     (- for covariant for short) `d1, -d1, d2, -d2, d3, -d3`
 
     `T^{d3 d2 d1}{}_{d1 d2 d3}` corresponds to `g = [4, 2, 0, 1, 3, 5, 6, 7]`
@@ -271,10 +280,13 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
     `T^{a0 a1 a2 a3 a4 a5} = -T^{a4 a1 a2 a3 a0 a5}`
 
     The dummy symmetry group D is generated by the strong base generators
-    `[(0, 1), (2, 3), (4, 5), (0, 1)(2, 3),(2, 3)(4, 5)]`
+    `[(0, 1), (2, 3), (4, 5), (0, 2)(1, 3), (0, 4)(1, 5)]`
+    where the first three interchange covariant and contravariant
+    positions of the same index (d1 <-> -d1) and the last two interchange
+    the dummy indices themselves (d1 <-> d2).
 
     The dummy symmetry acts from the left
-    `d = [1, 0, 2, 3, 4, 5, 6, 7]`  exchange `d1 -> -d1`
+    `d = [1, 0, 2, 3, 4, 5, 6, 7]`  exchange `d1 <-> -d1`
     `T^{d3 d2 d1}{}_{d1 d2 d3} == T^{d3 d2}{}_{d1}{}^{d1}{}_{d2 d3}`
 
     `g=[4, 2, 0, 1, 3, 5, 6, 7]  -> [4, 2, 1, 0, 3, 5, 6, 7] = _af_rmul(d, g)`
@@ -287,13 +299,13 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
     `g=[4,2,0,1,3,5,6,7]  -> [0, 2, 4, 1, 3, 5, 7, 6] = _af_rmul(g, s)`
 
     Example in which the tensor is zero, same slot symmetries as above:
-    `T^{d3}{}_{d1,d2}{}^{d1}{}_{d3}{}^{d2}`
+    `T^{d2}{}_{d1 d3}{}^{d1 d3}{}_{d2}`
 
-    `= -T^{d3}{}_{d1,d3}{}^{d1}{}_{d2}{}^{d2}`   under slot symmetry `-(2,4)`;
+    `= -T^{d3}{}_{d1 d3}{}^{d1 d2}{}_{d2}`   under slot symmetry `-(0,4)`;
 
-    `= T_{d3 d1}{}^{d3}{}^{d1}{}_{d2}{}^{d2}`    under slot symmetry `-(0,2)`;
+    `= T_{d3 d1}{}^{d3}{}^{d1 d2}{}_{d2}`    under slot symmetry `-(0,2)`;
 
-    `= T^{d3}{}_{d1 d3}{}^{d1}{}_{d2}{}^{d2}`    symmetric metric;
+    `= T^{d3}{}_{d1 d3}{}^{d1 d2}{}_{d2}`    symmetric metric;
 
     `= 0`  since two of these lines have tensors differ only for the sign.
 
@@ -409,7 +421,6 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
     dsgsx = []
     for i in range(num_types):
         dsgsx.extend(dummy_sgs(dumx[i], sym[i], num_dummies))
-    ginv = _af_invert(g)
     idn = list(range(size))
     # TAB = list of entries (s, d, h) where h = _af_rmuln(d,g,s)
     # for short, in the following d*g*s means _af_rmuln(d,g,s)
@@ -450,7 +461,6 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
         else:
             deltap = [p_i]
         TAB1 = []
-        nTAB = len(TAB)
         while TAB:
             s, d, h = TAB.pop()
             if min([md[h[x]] for x in deltab]) != p_i:
@@ -508,7 +518,6 @@ def double_coset_can_rep(dummies, sym, b_S, sgens, S_transversals, g):
         # if TAB contains equal permutations, keep only one of them;
         # if TAB contains equal permutations up to the sign, return 0
         TAB1.sort(key=lambda x: x[-1])
-        nTAB1 = len(TAB1)
         prev = [0] * size
         while TAB1:
             s, d, h = TAB1.pop()
@@ -581,13 +590,11 @@ def canonical_free(base, gens, g, num_free):
         return g[:]
 
     transversals = get_transversals(base, gens)
-    m = len(base)
     for x in sorted(g[:-2]):
         if x not in base:
             base.append(x)
     h = g
     for i, transv in enumerate(transversals):
-        b = base[i]
         h_i = [size]*num_free
         # find the element s in transversals[i] such that
         # _af_rmul(h, s) has its free elements with the lowest position in h
@@ -832,7 +839,6 @@ def canonicalize(g, dummies, msym, *v):
     size, sbase, sgens = gens_products(*v1)
 
     # reduce the permutations getting rid of the free indices
-    pos_dummies = [g1.index(x) for x in flat_dummies]
     pos_free = [g1.index(x) for x in range(num_free)]
     size_red = size - num_free
     g1_red = [x - num_free for x in g1 if x in flat_dummies]
@@ -894,7 +900,10 @@ def perm_af_direct_product(gens1, gens2, signed=True):
 
 def bsgs_direct_product(base1, gens1, base2, gens2, signed=True):
     """
-    direct product of two BSGS
+    Direct product of two BSGS
+
+    Parameters
+    ==========
 
     base1    base of the first BSGS.
 
