@@ -1,7 +1,7 @@
-from sympy.utilities.pytest import XFAIL, raises
+from sympy.utilities.pytest import XFAIL, raises, warns_deprecated_sympy
 from sympy import (S, Symbol, symbols, nan, oo, I, pi, Float, And, Or,
     Not, Implies, Xor, zoo, sqrt, Rational, simplify, Function,
-    log, cos, sin, Add, floor, ceiling)
+    log, cos, sin, Add, floor, ceiling, trigsimp)
 from sympy.core.compatibility import range
 from sympy.core.relational import (Relational, Equality, Unequality,
                                    GreaterThan, LessThan, StrictGreaterThan,
@@ -14,6 +14,35 @@ from itertools import combinations
 x, y, z, t = symbols('x,y,z,t')
 
 
+def rel_check(a, b):
+    from sympy.utilities.pytest import raises
+    assert a.is_number and b.is_number
+    for do in range(len(set([type(a), type(b)]))):
+        if S.NaN in (a, b):
+            v = [(a == b), (a != b)]
+            assert len(set(v)) == 1 and v[0] == False
+            assert not (a != b) and not (a == b)
+            assert raises(TypeError, lambda: a < b)
+            assert raises(TypeError, lambda: a <= b)
+            assert raises(TypeError, lambda: a > b)
+            assert raises(TypeError, lambda: a >= b)
+        else:
+            E = [(a == b), (a != b)]
+            assert len(set(E)) == 2
+            v = [
+            (a < b), (a <= b), (a > b), (a >= b)]
+            i = [
+            [True,    True,     False,   False],
+            [False,   True,     False,   True], # <-- i == 1
+            [False,   False,    True,    True]].index(v)
+            if i == 1:
+                assert E[0] or (a.is_Float != b.is_Float) # ugh
+            else:
+                assert E[1]
+        a, b = b, a
+    return True
+
+
 def test_rel_ne():
     assert Relational(x, y, '!=') == Ne(x, y)
 
@@ -87,11 +116,12 @@ def test_wrappers():
 
 
 def test_Eq():
-    assert Eq(x**2) == Eq(x**2, 0)
-    assert Eq(x**2) != Eq(x**2, 1)
 
     assert Eq(x, x)  # issue 5719
 
+    with warns_deprecated_sympy():
+        assert Eq(x) == Eq(x, 0)
+
     # issue 6116
     p = Symbol('p', positive=True)
     assert Eq(p, 0) is S.false
@@ -99,6 +129,9 @@ def test_Eq():
     # issue 13348
     assert Eq(True, 1) is S.false
 
+    assert Eq((), 1) is S.false
+
+
 def test_rel_Infinity():
     # NOTE: All of these are actually handled by sympy.core.Number, and do
     # not create Relational objects.
@@ -128,6 +161,50 @@ def test_rel_Infinity():
     assert (-oo <= 1) is S.true
 
 
+def test_infinite_symbol_inequalities():
+    x = Symbol('x', extended_positive=True, infinite=True)
+    y = Symbol('y', extended_positive=True, infinite=True)
+    z = Symbol('z', extended_negative=True, infinite=True)
+    w = Symbol('w', extended_negative=True, infinite=True)
+
+    inf_set = (x, y, oo)
+    ninf_set = (z, w, -oo)
+
+    for inf1 in inf_set:
+        assert (inf1 < 1) is S.false
+        assert (inf1 > 1) is S.true
+        assert (inf1 <= 1) is S.false
+        assert (inf1 >= 1) is S.true
+
+        for inf2 in inf_set:
+            assert (inf1 < inf2) is S.false
+            assert (inf1 > inf2) is S.false
+            assert (inf1 <= inf2) is S.true
+            assert (inf1 >= inf2) is S.true
+
+        for ninf1 in ninf_set:
+            assert (inf1 < ninf1) is S.false
+            assert (inf1 > ninf1) is S.true
+            assert (inf1 <= ninf1) is S.false
+            assert (inf1 >= ninf1) is S.true
+            assert (ninf1 < inf1) is S.true
+            assert (ninf1 > inf1) is S.false
+            assert (ninf1 <= inf1) is S.true
+            assert (ninf1 >= inf1) is S.false
+
+    for ninf1 in ninf_set:
+        assert (ninf1 < 1) is S.true
+        assert (ninf1 > 1) is S.false
+        assert (ninf1 <= 1) is S.true
+        assert (ninf1 >= 1) is S.false
+
+        for ninf2 in ninf_set:
+            assert (ninf1 < ninf2) is S.false
+            assert (ninf1 > ninf2) is S.false
+            assert (ninf1 <= ninf2) is S.true
+            assert (ninf1 >= ninf2) is S.true
+
+
 def test_bool():
     assert Eq(0, 0) is S.true
     assert Eq(1, 0) is S.false
@@ -183,18 +260,20 @@ def test_doit():
 def test_new_relational():
     x = Symbol('x')
 
-    assert Eq(x) == Relational(x, 0)       # None ==> Equality
-    assert Eq(x) == Relational(x, 0, '==')
-    assert Eq(x) == Relational(x, 0, 'eq')
-    assert Eq(x) == Equality(x, 0)
+    assert Eq(x, 0) == Relational(x, 0)       # None ==> Equality
+    assert Eq(x, 0) == Relational(x, 0, '==')
+    assert Eq(x, 0) == Relational(x, 0, 'eq')
+    assert Eq(x, 0) == Equality(x, 0)
+
+    assert Eq(x, 0) != Relational(x, 1)       # None ==> Equality
+    assert Eq(x, 0) != Relational(x, 1, '==')
+    assert Eq(x, 0) != Relational(x, 1, 'eq')
+    assert Eq(x, 0) != Equality(x, 1)
+
     assert Eq(x, -1) == Relational(x, -1)       # None ==> Equality
     assert Eq(x, -1) == Relational(x, -1, '==')
     assert Eq(x, -1) == Relational(x, -1, 'eq')
     assert Eq(x, -1) == Equality(x, -1)
-    assert Eq(x) != Relational(x, 1)       # None ==> Equality
-    assert Eq(x) != Relational(x, 1, '==')
-    assert Eq(x) != Relational(x, 1, 'eq')
-    assert Eq(x) != Equality(x, 1)
     assert Eq(x, -1) != Relational(x, 1)       # None ==> Equality
     assert Eq(x, -1) != Relational(x, 1, '==')
     assert Eq(x, -1) != Relational(x, 1, 'eq')
@@ -383,7 +462,7 @@ def test_imaginary_compare_raises_TypeError():
 def test_complex_compare_not_real():
     # two cases which are not real
     y = Symbol('y', imaginary=True)
-    z = Symbol('z', complex=True, real=False)
+    z = Symbol('z', complex=True, extended_real=False)
     for w in (y, z):
         assert_all_ineq_raise_TypeError(2, w)
     # some cases which should remain un-evaluated
@@ -455,7 +534,7 @@ def test_nan_equality_exceptions():
     assert Unequality(nan, nan) is S.true
 
     # See issue #7773
-    A = (x, S(0), S(1)/3, pi, oo, -oo)
+    A = (x, S.Zero, S.One/3, pi, oo, -oo)
     assert Equality(nan, random.choice(A)) is S.false
     assert Equality(random.choice(A), nan) is S.false
     assert Unequality(nan, random.choice(A)) is S.true
@@ -465,7 +544,7 @@ def test_nan_equality_exceptions():
 def test_nan_inequality_raise_errors():
     # See discussion in pull request #7776.  We test inequalities with
     # a set including examples of various classes.
-    for q in (x, S(0), S(10), S(1)/3, pi, S(1.3), oo, -oo, nan):
+    for q in (x, S.Zero, S(10), S.One/3, pi, S(1.3), oo, -oo, nan):
         assert_all_ineq_raise_TypeError(q, nan)
 
 
@@ -487,7 +566,7 @@ def test_inequalities_symbol_name_same():
     """Using the operator and functional forms should give same results."""
     # We test all combinations from a set
     # FIXME: could replace with random selection after test passes
-    A = (x, y, S(0), S(1)/3, pi, oo, -oo)
+    A = (x, y, S.Zero, S.One/3, pi, oo, -oo)
     for a in A:
         for b in A:
             assert Gt(a, b) == (a > b)
@@ -495,13 +574,13 @@ def test_inequalities_symbol_name_same():
             assert Ge(a, b) == (a >= b)
             assert Le(a, b) == (a <= b)
 
-    for b in (y, S(0), S(1)/3, pi, oo, -oo):
+    for b in (y, S.Zero, S.One/3, pi, oo, -oo):
         assert Gt(x, b, evaluate=False) == (x > b)
         assert Lt(x, b, evaluate=False) == (x < b)
         assert Ge(x, b, evaluate=False) == (x >= b)
         assert Le(x, b, evaluate=False) == (x <= b)
 
-    for b in (y, S(0), S(1)/3, pi, oo, -oo):
+    for b in (y, S.Zero, S.One/3, pi, oo, -oo):
         assert Gt(b, x, evaluate=False) == (b > x)
         assert Lt(b, x, evaluate=False) == (b < x)
         assert Ge(b, x, evaluate=False) == (b >= x)
@@ -513,7 +592,7 @@ def test_inequalities_symbol_name_same_complex():
     With complex non-real numbers, both should raise errors.
     """
     # FIXME: could replace with random selection after test passes
-    for a in (x, S(0), S(1)/3, pi, oo):
+    for a in (x, S.Zero, S.One/3, pi, oo, Rational(1, 3)):
         raises(TypeError, lambda: Gt(a, I))
         raises(TypeError, lambda: a > I)
         raises(TypeError, lambda: Lt(a, I))
@@ -530,7 +609,7 @@ def test_inequalities_cant_sympify_other():
 
     bar = "foo"
 
-    for a in (x, S(0), S(1)/3, pi, I, zoo, oo, -oo, nan):
+    for a in (x, S.Zero, S.One/3, pi, I, zoo, oo, -oo, nan, Rational(1, 3)):
         for op in (lt, gt, le, ge):
             raises(TypeError, lambda: op(a, bar))
 
@@ -552,30 +631,32 @@ def test_ineq_avoid_wild_symbol_flip():
 
 def test_issue_8245():
     a = S("6506833320952669167898688709329/5070602400912917605986812821504")
-    q = a.n(10)
-    assert (a == q) is True
-    assert (a != q) is False
-    assert (a > q) == False
-    assert (a < q) == False
-    assert (a >= q) == True
-    assert (a <= q) == True
+    assert rel_check(a, a.n(10))
+    assert rel_check(a, a.n(20))
+    assert rel_check(a, a.n())
+    # prec of 30 is enough to fully capture a as mpf
+    assert Float(a, 30) == Float(str(a.p), '')/Float(str(a.q), '')
+    for i in range(31):
+        r = Rational(Float(a, i))
+        f = Float(r)
+        assert (f < a) == (Rational(f) < a)
+    # test sign handling
+    assert (-f < -a) == (Rational(-f) < -a)
+    # test equivalence handling
+    isa = Float(a.p,'')/Float(a.q,'')
+    assert isa <= a
+    assert not isa < a
+    assert isa >= a
+    assert not isa > a
+    assert isa > 0
 
     a = sqrt(2)
     r = Rational(str(a.n(30)))
-    assert (r == a) is False
-    assert (r != a) is True
-    assert (r > a) == True
-    assert (r < a) == False
-    assert (r >= a) == True
-    assert (r <= a) == False
+    assert rel_check(a, r)
+
     a = sqrt(2)
     r = Rational(str(a.n(29)))
-    assert (r == a) is False
-    assert (r != a) is True
-    assert (r > a) == False
-    assert (r < a) == True
-    assert (r >= a) == False
-    assert (r <= a) == True
+    assert rel_check(a, r)
 
     assert Eq(log(cos(2)**2 + sin(2)**2), 0) == True
 
@@ -590,7 +671,9 @@ def test_issue_8449():
 
 def test_simplify_relational():
     assert simplify(x*(y + 1) - x*y - x + 1 < x) == (x > 1)
-    r = S(1) < x
+    assert simplify(x*(y + 1) - x*y - x - 1 < x) == (x > -1)
+    assert simplify(x < x*(y + 1) - x*y - x + 1) == (x < 1)
+    r = S.One < x
     # canonical operations are not the same as simplification,
     # so if there is no simplification, canonicalization will
     # be done unless the measure forbids it
@@ -600,17 +683,68 @@ def test_simplify_relational():
     # this will simplify to S.false and that is the
     # reason for the 'if r.is_Relational' in Relational's
     # _eval_simplify routine
-    assert simplify(-(2**(3*pi/2) + 6**pi)**(1/pi) +
+    assert simplify(-(2**(pi*Rational(3, 2)) + 6**pi)**(1/pi) +
                     2*(2**(pi/2) + 3**pi)**(1/pi) < 0) is S.false
     # canonical at least
-    for f in (Eq, Ne):
-        f(y, x).simplify() == f(x, y)
-        f(x - 1, 0).simplify() == f(x, 1)
-        f(x - 1, x).simplify() == S.false
-        f(2*x - 1, x).simplify() == f(x, 1)
-        f(2*x, 4).simplify() == f(x, 2)
-        z = cos(1)**2 + sin(1)**2 - 1  # z.is_zero is None
-        f(z*x, 0).simplify() == f(z*x, 0)
+    assert Eq(y, x).simplify() == Eq(x, y)
+    assert Eq(x - 1, 0).simplify() == Eq(x, 1)
+    assert Eq(x - 1, x).simplify() == S.false
+    assert Eq(2*x - 1, x).simplify() == Eq(x, 1)
+    assert Eq(2*x, 4).simplify() == Eq(x, 2)
+    z = cos(1)**2 + sin(1)**2 - 1  # z.is_zero is None
+    assert Eq(z*x, 0).simplify() == S.true
+
+    assert Ne(y, x).simplify() == Ne(x, y)
+    assert Ne(x - 1, 0).simplify() == Ne(x, 1)
+    assert Ne(x - 1, x).simplify() == S.true
+    assert Ne(2*x - 1, x).simplify() == Ne(x, 1)
+    assert Ne(2*x, 4).simplify() == Ne(x, 2)
+    assert Ne(z*x, 0).simplify() == S.false
+
+    # No real-valued assumptions
+    assert Ge(y, x).simplify() == Le(x, y)
+    assert Ge(x - 1, 0).simplify() == Ge(x, 1)
+    assert Ge(x - 1, x).simplify() == S.false
+    assert Ge(2*x - 1, x).simplify() == Ge(x, 1)
+    assert Ge(2*x, 4).simplify() == Ge(x, 2)
+    assert Ge(z*x, 0).simplify() == S.true
+    assert Ge(x, -2).simplify() == Ge(x, -2)
+    assert Ge(-x, -2).simplify() == Le(x, 2)
+    assert Ge(x, 2).simplify() == Ge(x, 2)
+    assert Ge(-x, 2).simplify() == Le(x, -2)
+
+    assert Le(y, x).simplify() == Ge(x, y)
+    assert Le(x - 1, 0).simplify() == Le(x, 1)
+    assert Le(x - 1, x).simplify() == S.true
+    assert Le(2*x - 1, x).simplify() == Le(x, 1)
+    assert Le(2*x, 4).simplify() == Le(x, 2)
+    assert Le(z*x, 0).simplify() == S.true
+    assert Le(x, -2).simplify() == Le(x, -2)
+    assert Le(-x, -2).simplify() == Ge(x, 2)
+    assert Le(x, 2).simplify() == Le(x, 2)
+    assert Le(-x, 2).simplify() == Ge(x, -2)
+
+    assert Gt(y, x).simplify() == Lt(x, y)
+    assert Gt(x - 1, 0).simplify() == Gt(x, 1)
+    assert Gt(x - 1, x).simplify() == S.false
+    assert Gt(2*x - 1, x).simplify() == Gt(x, 1)
+    assert Gt(2*x, 4).simplify() == Gt(x, 2)
+    assert Gt(z*x, 0).simplify() == S.false
+    assert Gt(x, -2).simplify() == Gt(x, -2)
+    assert Gt(-x, -2).simplify() == Lt(x, 2)
+    assert Gt(x, 2).simplify() == Gt(x, 2)
+    assert Gt(-x, 2).simplify() == Lt(x, -2)
+
+    assert Lt(y, x).simplify() == Gt(x, y)
+    assert Lt(x - 1, 0).simplify() == Lt(x, 1)
+    assert Lt(x - 1, x).simplify() == S.true
+    assert Lt(2*x - 1, x).simplify() == Lt(x, 1)
+    assert Lt(2*x, 4).simplify() == Lt(x, 2)
+    assert Lt(z*x, 0).simplify() == S.false
+    assert Lt(x, -2).simplify() == Lt(x, -2)
+    assert Lt(-x, -2).simplify() == Gt(x, 2)
+    assert Lt(x, 2).simplify() == Lt(x, 2)
+    assert Lt(-x, 2).simplify() == Gt(x, -2)
 
 
 def test_equals():
@@ -695,6 +829,14 @@ def test_issue_10401():
     fin = symbols('inf', finite=True)
     inf = symbols('inf', infinite=True)
     inf2 = symbols('inf2', infinite=True)
+    infx = symbols('infx', infinite=True, extended_real=True)
+    # Used in the commented tests below:
+    #infx2 = symbols('infx2', infinite=True, extended_real=True)
+    infnx = symbols('inf~x', infinite=True, extended_real=False)
+    infnx2 = symbols('inf~x2', infinite=True, extended_real=False)
+    infp = symbols('infp', infinite=True, extended_positive=True)
+    infp1 = symbols('infp1', infinite=True, extended_positive=True)
+    infn = symbols('infn', infinite=True, extended_negative=True)
     zero = symbols('z', zero=True)
     nonzero = symbols('nz', zero=False, finite=True)
 
@@ -704,12 +846,36 @@ def test_issue_10401():
 
     T, F = S.true, S.false
     assert Eq(fin, inf) is F
-    assert Eq(inf, inf2) is T and inf != inf2
+    assert Eq(inf, inf2) not in (T, F) and inf != inf2
+    assert Eq(1 + inf, 2 + inf2) not in (T, F) and inf != inf2
+    assert Eq(infp, infp1) is T
+    assert Eq(infp, infn) is F
+    assert Eq(1 + I*oo, I*oo) is F
+    assert Eq(I*oo, 1 + I*oo) is F
+    assert Eq(1 + I*oo, 2 + I*oo) is F
+    assert Eq(1 + I*oo, 2 + I*infx) is F
+    assert Eq(1 + I*oo, 2 + infx) is F
+    # FIXME: The test below fails because (-infx).is_extended_positive is True
+    # (should be None)
+    #assert Eq(1 + I*infx, 1 + I*infx2) not in (T, F) and infx != infx2
+    #
+    assert Eq(zoo, sqrt(2) + I*oo) is F
+    assert Eq(zoo, oo) is F
+    r = Symbol('r', real=True)
+    i = Symbol('i', imaginary=True)
+    assert Eq(i*I, r) not in (T, F)
+    assert Eq(infx, infnx) is F
+    assert Eq(infnx, infnx2) not in (T, F) and infnx != infnx2
+    assert Eq(zoo, oo) is F
     assert Eq(inf/inf2, 0) is F
     assert Eq(inf/fin, 0) is F
     assert Eq(fin/inf, 0) is T
     assert Eq(zero/nonzero, 0) is T and ((zero/nonzero) != 0)
-    assert Eq(inf, -inf) is F
+    # The commented out test below is incorrect because:
+    assert zoo == -zoo
+    assert Eq(zoo, -zoo) is T
+    assert Eq(oo, -oo) is F
+    assert Eq(inf, -inf) not in (T, F)
 
     assert Eq(fin/(fin + 1), 1) is S.false
 
@@ -755,22 +921,11 @@ def test_issues_13081_12583_12534():
     # pick one such precision and affirm that the reversed operation
     # gives the opposite result, i.e. if x < y is true then x > y
     # must be false
-    p = 20
-    # Rational vs NumberSymbol
-    G = [Rational(pi.n(i)) > pi for i in (p, p + 1)]
-    L = [Rational(pi.n(i)) < pi for i in (p, p + 1)]
-    assert G == [False, True]
-    assert all(i is not j for i, j in zip(L, G))
-    # Float vs NumberSymbol
-    G = [pi.n(i) > pi for i in (p, p + 1)]
-    L = [pi.n(i) < pi for i in (p, p + 1)]
-    assert G == [False, True]
-    assert all(i is not j for i, j in zip(L, G))
-    # Float vs Float
-    G = [pi.n(p) > pi.n(p + 1)]
-    L = [pi.n(p) < pi.n(p + 1)]
-    assert G == [True]
-    assert all(i is not j for i, j in zip(L, G))
+    for i in (20, 21):
+        v = pi.n(i)
+        assert rel_check(Rational(v), pi)
+        assert rel_check(v, pi)
+    assert rel_check(pi.n(20), pi.n(21))
     # Float vs Rational
     # the rational form is less than the floating representation
     # at the same precision
@@ -796,7 +951,7 @@ def test_rel_args():
     # can be removed.
     for op in ['<', '<=', '>', '>=']:
         for b in (S.true, x < 1, And(x, y)):
-            for v in (0.1, 1, 2**32, t, S(1)):
+            for v in (0.1, 1, 2**32, t, S.One):
                 raises(TypeError, lambda: Relational(b, v, op))
 
 
@@ -881,3 +1036,48 @@ def test_improved_canonical():
     test_different_forms(generate_forms(pi - 5*y < -x + 2*y**2 - 7))
 
     assert (pi >= x).canonical == (x <= pi)
+
+
+def test_set_equality_canonical():
+    a, b, c = symbols('a b c')
+
+    A = Eq(FiniteSet(a, b, c), FiniteSet(1, 2, 3))
+    B = Ne(FiniteSet(a, b, c), FiniteSet(4, 5, 6))
+
+    assert A.canonical == A.reversed
+    assert B.canonical == B.reversed
+
+
+def test_trigsimp():
+    # issue 16736
+    s, c = sin(2*x), cos(2*x)
+    eq = Eq(s, c)
+    assert trigsimp(eq) == eq  # no rearrangement of sides
+    # simplification of sides might result in
+    # an unevaluated Eq
+    changed = trigsimp(Eq(s + c, sqrt(2)))
+    assert isinstance(changed, Eq)
+    assert changed.subs(x, pi/8) is S.true
+    # or an evaluated one
+    assert trigsimp(Eq(cos(x)**2 + sin(x)**2, 1)) is S.true
+
+
+def test_polynomial_relation_simplification():
+    assert Ge(3*x*(x + 1) + 4, 3*x).simplify() in [Ge(x**2, -Rational(4,3)), Le(-x**2, Rational(4, 3))]
+    assert Le(-(3*x*(x + 1) + 4), -3*x).simplify() in [Ge(x**2, -Rational(4,3)), Le(-x**2, Rational(4, 3))]
+    assert ((x**2+3)*(x**2-1)+3*x >= 2*x**2).simplify() in [(x**4 + 3*x >= 3), (-x**4 - 3*x <= -3)]
+
+
+def test_multivariate_linear_function_simplification():
+    assert Ge(x + y, x - y).simplify() == Ge(y, 0)
+    assert Le(-x + y, -x - y).simplify() == Le(y, 0)
+    assert Eq(2*x + y, 2*x + y - 3).simplify() == False
+    assert (2*x + y > 2*x + y - 3).simplify() == True
+    assert (2*x + y < 2*x + y - 3).simplify() == False
+    assert (2*x + y < 2*x + y + 3).simplify() == True
+    a, b, c, d, e, f, g = symbols('a b c d e f g')
+    assert Lt(a + b + c + 2*d, 3*d - f + g). simplify() == Lt(a, -b - c + d - f + g)
+
+
+def test_nonpolymonial_relations():
+    assert Eq(cos(x), 0).simplify() == Eq(cos(x), 0)
