@@ -30,8 +30,10 @@ from sympy.core.symbol import Dummy
 
 from sympy.polys import Poly, gcd, ZZ, cancel
 
+from sympy import sqrt, re, im
+
 from sympy.integrals.risch import (gcdex_diophantine, frac_in, derivation,
-    splitfactor, NonElementaryIntegralException, DecrementLevel)
+    splitfactor, NonElementaryIntegralException, DecrementLevel, recognize_log_derivative)
 
 # TODO: Add messages to NonElementaryIntegralException errors
 
@@ -221,8 +223,8 @@ def special_denom(a, ba, bd, ca, cd, DE, case='auto'):
                 etaa, etad = frac_in(dcoeff, DE.t)
                 A = parametric_log_deriv(alphaa, alphad, etaa, etad, DE)
                 if A is not None:
-                    a, m, z = A
-                    if a == 1:
+                    Q, m, z = A
+                    if Q == 1:
                         n = min(n, m)
 
         elif case == 'tan':
@@ -236,8 +238,8 @@ def special_denom(a, ba, bd, ca, cd, DE, case='auto'):
                 if recognize_log_derivative(2*betaa, betad, DE):
                     A = parametric_log_deriv(alphaa*sqrt(-1)*betad+alphad*betaa, alphad*betad, etaa, etad, DE)
                     if A is not None:
-                       a, m, z = A
-                       if a == 1:
+                       Q, m, z = A
+                       if Q == 1:
                            n = min(n, m)
     N = max(0, -nb, n - nc)
     pN = p**N
@@ -713,7 +715,7 @@ def solve_poly_rde(b, cQ, n, DE, parametric=False):
 
             else:
                 raise NotImplementedError("Other Poly (P)RDE cancellation "
-                    "cases are not yet implemented (%s)." % case)
+                    "cases are not yet implemented (%s)." % DE.case)
 
         if parametric:
             raise NotImplementedError("Remaining cases for Poly PRDE not yet "
