@@ -2,24 +2,30 @@ from sympy import (
     Abs, acos, acosh, Add, And, asin, asinh, atan, Ci, cos, sinh, cosh,
     tanh, Derivative, diff, DiracDelta, E, Ei, Eq, exp, erf, erfc, erfi,
     EulerGamma, Expr, factor, Function, gamma, gammasimp, I, Idx, im, IndexedBase,
-    Integral, integrate, Interval, Lambda, LambertW, log, Matrix, Max, meijerg, Min, nan,
+    integrate, Interval, Lambda, LambertW, log, Matrix, Max, meijerg, Min, nan,
     Ne, O, oo, pi, Piecewise, polar_lift, Poly, polygamma, Rational, re, S, Si, sign,
     simplify, sin, sinc, SingularityFunction, sqrt, sstr, Sum, Symbol,
-    symbols, sympify, tan, trigsimp, Tuple
+    symbols, sympify, tan, trigsimp, Tuple, lerchphi, exp_polar, li, hyper
 )
+from sympy.core.compatibility import range
+from sympy.core.expr import unchanged
 from sympy.functions.elementary.complexes import periodic_argument
 from sympy.functions.elementary.integers import floor
+from sympy.integrals.integrals import Integral
 from sympy.integrals.risch import NonElementaryIntegral
 from sympy.physics import units
-from sympy.core.compatibility import range
-from sympy.utilities.pytest import XFAIL, raises, slow, skip, ON_TRAVIS
+from sympy.utilities.pytest import raises, slow, skip, ON_TRAVIS
 from sympy.utilities.randtest import verify_numerically
-from sympy.integrals.integrals import Integral
 
-x, y, a, t, x_1, x_2, z, s, b= symbols('x y a t x_1 x_2 z s b')
+x, y, a, t, x_1, x_2, z, s, b = symbols('x y a t x_1 x_2 z s b')
 n = Symbol('n', integer=True)
 f = Function('f')
 
+
+def NS(e, n=15, **options):
+    return sstr(sympify(e).evalf(n, **options), full_prec=True)
+
+
 def test_principal_value():
     g = 1 / x
     assert Integral(g, (x, -oo, oo)).principal_value() == 0
@@ -40,13 +46,14 @@ def test_principal_value():
     assert Integral(v, (x, -2, 2)).principal_value() == 0
 
     s = x ** 2 / (x ** 2 - 1)
-    assert Integral(s, (x, -oo, oo)).principal_value() == oo
+    assert Integral(s, (x, -oo, oo)).principal_value() is oo
     assert Integral(s, (x, -2, 2)).principal_value() == -log(3) + 4
 
     f = 1 / ((x ** 2 - 1) * (1 + x ** 2))
     assert Integral(f, (x, -oo, oo)).principal_value() == -pi / 2
     assert Integral(f, (x, -2, 2)).principal_value() == -atan(2) - log(3) / 2
 
+
 def diff_test(i):
     """Return the set of symbols, s, which were used in testing that
     i.diff(s) agrees with i.doit().diff(s). If there is an error then
@@ -90,7 +97,7 @@ def test_basics():
     assert Integral(0, x) != 0
     assert Integral(x, (x, 1, 1)) != 0
     assert Integral(oo, x) != oo
-    assert Integral(S.NaN, x) == S.NaN
+    assert Integral(S.NaN, x) is S.NaN
 
     assert diff(Integral(y, y), x) == 0
     assert diff(Integral(x, (x, 0, 1)), x) == 0
@@ -115,7 +122,7 @@ def test_basics():
     assert integrate(t**2, (t, x, 2*x)).diff(x) == 7*x**2
 
     assert Integral(x, x).atoms() == {x}
-    assert Integral(f(x), (x, 0, 1)).atoms() == {S(0), S(1), x}
+    assert Integral(f(x), (x, 0, 1)).atoms() == {S.Zero, S.One, x}
 
     assert diff_test(Integral(x, (x, 3*y))) == {y}
     assert diff_test(Integral(x, (a, 3*y))) == {x, y}
@@ -138,7 +145,7 @@ def test_diff_wrt():
 
     t = Test()
     assert integrate(t + 1, t) == t**2/2 + t
-    assert integrate(t + 1, (t, 0, 1)) == S(3)/2
+    assert integrate(t + 1, (t, 0, 1)) == Rational(3, 2)
 
     raises(ValueError, lambda: integrate(x + 1, x + 1))
     raises(ValueError, lambda: integrate(x + 1, (x + 1, 0, 1)))
@@ -194,7 +201,7 @@ def test_multiple_integration():
     assert integrate((x**2)*(y**2), (x, 0, 1), (y, -1, 2)) == Rational(1)
     assert integrate((y**2)*(x**2), x, y) == Rational(1, 9)*(x**3)*(y**3)
     assert integrate(1/(x + 3)/(1 + x)**3, x) == \
-        -S(1)/8*log(3 + x) + S(1)/8*log(1 + x) + x/(4 + 8*x + 4*x**2)
+        log(3 + x)*Rational(-1, 8) + log(1 + x)*Rational(1, 8) + x/(4 + 8*x + 4*x**2)
     assert integrate(sin(x*y)*y, (x, 0, 1), (y, 0, 1)) == -sin(1) + 1
 
 
@@ -236,7 +243,7 @@ def test_integrate_poly_defined():
     assert Qx.gens == (y,)
     assert Qy.gens == (x,)
 
-    assert Qx.as_expr() == Rational(1, 2) + y/3 + y**3
+    assert Qx.as_expr() == S.Half + y/3 + y**3
     assert Qy.as_expr() == pi**4/4 + pi*x + pi**2*x**2/2
 
 
@@ -294,8 +301,8 @@ def test_issue_3623():
 def test_issue_3664():
     n = Symbol('n', integer=True, nonzero=True)
     assert integrate(-1./2 * x * sin(n * pi * x/2), [x, -2, 0]) == \
-        2*cos(pi*n)/(pi*n)
-    assert integrate(-Rational(1)/2 * x * sin(n * pi * x/2), [x, -2, 0]) == \
+        2.0*cos(pi*n)/(pi*n)
+    assert integrate(x * sin(n * pi * x/2) * Rational(-1, 2), [x, -2, 0]) == \
         2*cos(pi*n)/(pi*n)
 
 
@@ -309,6 +316,7 @@ def test_issue_3686():  # remove this when fresnel itegrals are implemented
     assert expand_func(integrate(sin(x**2), x)) == \
         sqrt(2)*sqrt(pi)*fresnels(sqrt(2)*x/sqrt(pi))/2
 
+
 def test_integrate_units():
     m = units.m
     s = units.s
@@ -347,7 +355,7 @@ def test_issue_4516():
 
 def test_issue_7450():
     ans = integrate(exp(-(1 + I)*x), (x, 0, oo))
-    assert re(ans) == S.Half and im(ans) == -S.Half
+    assert re(ans) == S.Half and im(ans) == Rational(-1, 2)
 
 
 def test_issue_8623():
@@ -417,17 +425,34 @@ def test_transform():
     assert i.transform(x, (x + 2*y, x)).doit() == \
         i.transform(x, (x + 2*z, x)).doit() == 3
 
+    i = Integral(x, (x, a, b))
+    assert i.transform(x, 2*s) == Integral(4*s, (s, a/2, b/2))
+    raises(ValueError, lambda: i.transform(x, 1))
+    raises(ValueError, lambda: i.transform(x, s*t))
+    raises(ValueError, lambda: i.transform(x, -s))
+    raises(ValueError, lambda: i.transform(x, (s, t)))
+    raises(ValueError, lambda: i.transform(2*x, 2*s))
+
+    i = Integral(x**2, (x, 1, 2))
+    raises(ValueError, lambda: i.transform(x**2, s))
+
+    am = Symbol('a', negative=True)
+    bp = Symbol('b', positive=True)
+    i = Integral(x, (x, bp, am))
+    i.transform(x, 2*s)
+    assert i.transform(x, 2*s) == Integral(-4*s, (s, am/2, bp/2))
+
+    i = Integral(x, (x, a))
+    assert i.transform(x, 2*s) == Integral(4*s, (s, a/2))
+
 
 def test_issue_4052():
-    f = S(1)/2*asin(x) + x*sqrt(1 - x**2)/2
+    f = S.Half*asin(x) + x*sqrt(1 - x**2)/2
 
     assert integrate(cos(asin(x)), x) == f
     assert integrate(sin(acos(x)), x) == f
 
 
-def NS(e, n=15, **options):
-    return sstr(sympify(e).evalf(n, **options), full_prec=True)
-
 
 @slow
 def test_evalf_integrals():
@@ -496,14 +521,15 @@ def test_evalf_issue_939():
         integrate(1/(x**5 + 1), (x, 2, 4)), chop=True) == '0.0144361088886740'
 
 
-@XFAIL
-def test_failing_integrals():
-    #---
-    # Double integrals not implemented
-    assert NS(Integral(
-        sqrt(x) + x*y, (x, 1, 2), (y, -1, 1)), 15) == '2.43790283299492'
+def test_double_previously_failing_integrals():
+    # Double integrals not implemented <- Sure it is!
+    res = integrate(sqrt(x) + x*y, (x, 1, 2), (y, -1, 1))
+    # Old numerical test
+    assert NS(res, 15) == '2.43790283299492'
+    # Symbolic test
+    assert res == Rational(-4, 3) + 8*sqrt(2)/3
     # double integral + zero detection
-    assert NS(Integral(sin(x + x*y), (x, -1, 1), (y, -1, 1)), 15) == '0.0'
+    assert integrate(sin(x + x*y), (x, -1, 1), (y, -1, 1)) is S.Zero
 
 
 def test_integrate_SingularityFunction():
@@ -521,7 +547,8 @@ def test_integrate_SingularityFunction():
     assert integrate(in_3, x) == out_3_1
     assert integrate(in_3, y) == out_3_2
 
-    assert Integral(in_3, x) == Integral(in_3, x)
+    assert unchanged(Integral, in_3, (x,))
+    assert Integral(in_3, x) == Integral(in_3, (x,))
     assert Integral(in_3, x).doit() == out_3_1
 
     in_4 = 10*SingularityFunction(x, -4, 7) - 2*SingularityFunction(x, 10, -2)
@@ -551,13 +578,6 @@ def test_integrate_DiracDelta():
         1/sqrt(101*pi)
 
 
-@XFAIL
-def test_integrate_DiracDelta_fails():
-    # issue 6427
-    assert integrate(integrate(integrate(
-        DiracDelta(x - y - z), (z, 0, oo)), (y, 0, 1)), (x, 0, 1)) == S(1)/2
-
-
 def test_integrate_returns_piecewise():
     assert integrate(x**y, x) == Piecewise(
         (x**(y + 1)/(y + 1), Ne(y, -1)), (log(x), True))
@@ -586,14 +606,15 @@ def test_integrate_returns_piecewise():
     assert integrate(exp(x*y), (x, 0, z)) == Piecewise(
         (exp(y*z)/y - 1/y, (y > -oo) & (y < oo) & Ne(y, 0)), (z, True))
 
+
 def test_integrate_max_min():
     x = symbols('x', real=True)
     assert integrate(Min(x, 2), (x, 0, 3)) == 4
-    assert integrate(Max(x**2, x**3), (x, 0, 2)) == S(49)/12
+    assert integrate(Max(x**2, x**3), (x, 0, 2)) == Rational(49, 12)
     assert integrate(Min(exp(x), exp(-x))**2, x) == Piecewise( \
         (exp(2*x)/2, x <= 0), (1 - exp(-2*x)/2, True))
     # issue 7907
-    c = symbols('c', real=True)
+    c = symbols('c', extended_real=True)
     int1 = integrate(Max(c, x)*exp(-x**2), (x, -oo, oo))
     int2 = integrate(c*exp(-x**2), (x, -oo, c))
     int3 = integrate(x*exp(-x**2), (x, c, oo))
@@ -602,14 +623,14 @@ def test_integrate_max_min():
 
 
 def test_integrate_Abs_sign():
-    assert integrate(Abs(x), (x, -2, 1)) == S(5)/2
-    assert integrate(Abs(x), (x, 0, 1)) == S(1)/2
-    assert integrate(Abs(x + 1), (x, 0, 1)) == S(3)/2
+    assert integrate(Abs(x), (x, -2, 1)) == Rational(5, 2)
+    assert integrate(Abs(x), (x, 0, 1)) == S.Half
+    assert integrate(Abs(x + 1), (x, 0, 1)) == Rational(3, 2)
     assert integrate(Abs(x**2 - 1), (x, -2, 2)) == 4
     assert integrate(Abs(x**2 - 3*x), (x, -15, 15)) == 2259
     assert integrate(sign(x), (x, -1, 2)) == 1
     assert integrate(sign(x)*sin(x), (x, -pi, pi)) == 4
-    assert integrate(sign(x - 2) * x**2, (x, 0, 3)) == S(11)/3
+    assert integrate(sign(x - 2) * x**2, (x, 0, 3)) == Rational(11, 3)
 
     t, s = symbols('t s', real=True)
     assert integrate(Abs(t), t) == Piecewise(
@@ -708,6 +729,7 @@ def test_expand():
     e = Integral(f(x)+f(x**2), (x, 1, y))
     assert e.expand() == Integral(f(x), (x, 1, y)) + Integral(f(x**2), (x, 1, y))
 
+
 def test_integration_variable():
     raises(ValueError, lambda: Integral(exp(-x**2), 3))
     raises(ValueError, lambda: Integral(exp(-x**2), (3, -oo, oo)))
@@ -739,43 +761,45 @@ def test_as_sum_midpoint1():
 def test_as_sum_midpoint2():
     e = Integral((x + y)**2, (x, 0, 1))
     n = Symbol('n', positive=True, integer=True)
-    assert e.as_sum(1, method="midpoint").expand() == S(1)/4 + y + y**2
-    assert e.as_sum(2, method="midpoint").expand() == S(5)/16 + y + y**2
-    assert e.as_sum(3, method="midpoint").expand() == S(35)/108 + y + y**2
-    assert e.as_sum(4, method="midpoint").expand() == S(21)/64 + y + y**2
+    assert e.as_sum(1, method="midpoint").expand() == Rational(1, 4) + y + y**2
+    assert e.as_sum(2, method="midpoint").expand() == Rational(5, 16) + y + y**2
+    assert e.as_sum(3, method="midpoint").expand() == Rational(35, 108) + y + y**2
+    assert e.as_sum(4, method="midpoint").expand() == Rational(21, 64) + y + y**2
     assert e.as_sum(n, method="midpoint").expand() == \
-        y**2 + y + S(1)/3 - 1/(12*n**2)
+        y**2 + y + Rational(1, 3) - 1/(12*n**2)
 
 
 def test_as_sum_left():
     e = Integral((x + y)**2, (x, 0, 1))
     assert e.as_sum(1, method="left").expand() == y**2
-    assert e.as_sum(2, method="left").expand() == S(1)/8 + y/2 + y**2
-    assert e.as_sum(3, method="left").expand() == S(5)/27 + 2*y/3 + y**2
-    assert e.as_sum(4, method="left").expand() == S(7)/32 + 3*y/4 + y**2
+    assert e.as_sum(2, method="left").expand() == Rational(1, 8) + y/2 + y**2
+    assert e.as_sum(3, method="left").expand() == Rational(5, 27) + y*Rational(2, 3) + y**2
+    assert e.as_sum(4, method="left").expand() == Rational(7, 32) + y*Rational(3, 4) + y**2
     assert e.as_sum(n, method="left").expand() == \
-        y**2 + y + S(1)/3 - y/n - 1/(2*n) + 1/(6*n**2)
+        y**2 + y + Rational(1, 3) - y/n - 1/(2*n) + 1/(6*n**2)
     assert e.as_sum(10, method="left", evaluate=False).has(Sum)
 
+
 def test_as_sum_right():
     e = Integral((x + y)**2, (x, 0, 1))
     assert e.as_sum(1, method="right").expand() == 1 + 2*y + y**2
-    assert e.as_sum(2, method="right").expand() == S(5)/8 + 3*y/2 + y**2
-    assert e.as_sum(3, method="right").expand() == S(14)/27 + 4*y/3 + y**2
-    assert e.as_sum(4, method="right").expand() == S(15)/32 + 5*y/4 + y**2
+    assert e.as_sum(2, method="right").expand() == Rational(5, 8) + y*Rational(3, 2) + y**2
+    assert e.as_sum(3, method="right").expand() == Rational(14, 27) + y*Rational(4, 3) + y**2
+    assert e.as_sum(4, method="right").expand() == Rational(15, 32) + y*Rational(5, 4) + y**2
     assert e.as_sum(n, method="right").expand() == \
-        y**2 + y + S(1)/3 + y/n + 1/(2*n) + 1/(6*n**2)
+        y**2 + y + Rational(1, 3) + y/n + 1/(2*n) + 1/(6*n**2)
 
 
 def test_as_sum_trapezoid():
     e = Integral((x + y)**2, (x, 0, 1))
-    assert e.as_sum(1, method="trapezoid").expand() == y**2 + y + S(1)/2
-    assert e.as_sum(2, method="trapezoid").expand() == y**2 + y + S(3)/8
-    assert e.as_sum(3, method="trapezoid").expand() == y**2 + y + S(19)/54
-    assert e.as_sum(4, method="trapezoid").expand() == y**2 + y + S(11)/32
+    assert e.as_sum(1, method="trapezoid").expand() == y**2 + y + S.Half
+    assert e.as_sum(2, method="trapezoid").expand() == y**2 + y + Rational(3, 8)
+    assert e.as_sum(3, method="trapezoid").expand() == y**2 + y + Rational(19, 54)
+    assert e.as_sum(4, method="trapezoid").expand() == y**2 + y + Rational(11, 32)
     assert e.as_sum(n, method="trapezoid").expand() == \
-        y**2 + y + S(1)/3 + 1/(6*n**2)
-    assert Integral(sign(x), (x, 0, 1)).as_sum(1, 'trapezoid') == S(1)/2
+        y**2 + y + Rational(1, 3) + 1/(6*n**2)
+    assert Integral(sign(x), (x, 0, 1)).as_sum(1, 'trapezoid') == S.Half
+
 
 def test_as_sum_raises():
     e = Integral((x + y)**2, (x, 0, 1))
@@ -821,7 +845,7 @@ def test_doit_integrals():
     assert Integral(0, (x, 1, Integral(f(x), x))).doit() == 0
     assert Integral(x, (a, 0)).doit() == 0
     limits = ((a, 1, exp(x)), (x, 0))
-    assert Integral(a, *limits).doit() == S(1)/4
+    assert Integral(a, *limits).doit() == Rational(1, 4)
     assert Integral(a, *list(reversed(limits))).doit() == 0
 
 
@@ -977,13 +1001,17 @@ def test_issue_5167():
 def test_issue_4890():
     z = Symbol('z', positive=True)
     assert integrate(exp(-log(x)**2), x) == \
-        sqrt(pi)*exp(S(1)/4)*erf(log(x)-S(1)/2)/2
+        sqrt(pi)*exp(Rational(1, 4))*erf(log(x) - S.Half)/2
     assert integrate(exp(log(x)**2), x) == \
-        sqrt(pi)*exp(-S(1)/4)*erfi(log(x)+S(1)/2)/2
+        sqrt(pi)*exp(Rational(-1, 4))*erfi(log(x)+S.Half)/2
     assert integrate(exp(-z*log(x)**2), x) == \
         sqrt(pi)*exp(1/(4*z))*erf(sqrt(z)*log(x) - 1/(2*sqrt(z)))/(2*sqrt(z))
 
 
+def test_issue_4551():
+    assert not integrate(1/(x*sqrt(1 - x**2)), x).has(Integral)
+
+
 def test_issue_4376():
     n = Symbol('n', integer=True, positive=True)
     assert simplify(integrate(n*(x**(1/n) - 1), (x, 0, S.Half)) -
@@ -997,12 +1025,17 @@ def test_issue_4517():
 
 def test_issue_4527():
     k, m = symbols('k m', integer=True)
-    ans = integrate(sin(k*x)*sin(m*x), (x, 0, pi)
-            ).simplify() == Piecewise(
-        (0, Eq(k, 0) | Eq(m, 0)),
-        (-pi/2, Eq(k, -m)),
-        (pi/2, Eq(k, m)),
-        (0, True))
+    assert integrate(sin(k*x)*sin(m*x), (x, 0, pi)).simplify() == \
+        Piecewise((0, Eq(k, 0) | Eq(m, 0)),
+                  (-pi/2, Eq(k, -m) | (Eq(k, 0) & Eq(m, 0))),
+                  (pi/2, Eq(k, m) | (Eq(k, 0) & Eq(m, 0))),
+                  (0, True))
+    # Should be possible to further simplify to:
+    # Piecewise(
+    #    (0, Eq(k, 0) | Eq(m, 0)),
+    #    (-pi/2, Eq(k, -m)),
+    #    (pi/2, Eq(k, m)),
+    #    (0, True))
     assert integrate(sin(k*x)*sin(m*x), (x,)) == Piecewise(
         (0, And(Eq(k, 0), Eq(m, 0))),
         (-x*sin(m*x)**2/2 - x*cos(m*x)**2/2 + sin(m*x)*cos(m*x)/(2*m), Eq(k, -m)),
@@ -1099,8 +1132,7 @@ def test_atom_bug():
 def test_limit_bug():
     z = Symbol('z', zero=False)
     assert integrate(sin(x*y*z), (x, 0, pi), (y, 0, pi)) == \
-        (log(z**2) + 2*EulerGamma + 2*log(pi))/(2*z) - \
-        (-log(pi*z) + log(pi**2*z**2)/2 + Ci(pi**2*z))/z + log(pi)/z
+        (log(z) + EulerGamma + log(pi))/z - Ci(pi**2*z)/z + log(pi)/z
 
 
 def test_issue_4703():
@@ -1133,6 +1165,7 @@ def test_issue_4493():
 def test_issue_4737():
     assert integrate(sin(x)/x, (x, -oo, oo)) == pi
     assert integrate(sin(x)/x, (x, 0, oo)) == pi/2
+    assert integrate(sin(x)/x, x) == Si(x)
 
 
 def test_issue_4992():
@@ -1150,7 +1183,7 @@ def test_issue_4487():
 
 def test_issue_4215():
     x = Symbol("x")
-    assert integrate(1/(x**2), (x, -1, 1)) == oo
+    assert integrate(1/(x**2), (x, -1, 1)) is oo
 
 
 def test_issue_4400():
@@ -1178,7 +1211,10 @@ def test_issue_4326():
     R, b, h = symbols('R b h')
     # It doesn't matter if we can do the integral.  Just make sure the result
     # doesn't contain nan.  This is really a test against _eval_interval.
-    assert not integrate(((h*(x - R + b))/b)*sqrt(R**2 - x**2), (x, R - b, R)).has(nan)
+    e = integrate(((h*(x - R + b))/b)*sqrt(R**2 - x**2), (x, R - b, R))
+    assert not e.has(nan)
+    # See that it evaluates
+    assert not e.has(Integral)
 
 
 def test_powers():
@@ -1213,9 +1249,9 @@ def test_heurisch_option():
     # an integral that heurisch currently cannot handle
     assert integrate(exp(x)/x, x, heurisch=True) == Integral(exp(x)/x, x)
     # an integral where heurisch currently hangs, issue 15471
-    assert integrate(log(x)*cos(log(x))/x**(S(3)/4), x, heurisch=False) == (
-        -128*x**(S(1)/4)*sin(log(x))/289 + 240*x**(S(1)/4)*cos(log(x))/289 +
-        (16*x**(S(1)/4)*sin(log(x))/17 + 4*x**(S(1)/4)*cos(log(x))/17)*log(x))
+    assert integrate(log(x)*cos(log(x))/x**Rational(3, 4), x, heurisch=False) == (
+        -128*x**Rational(1, 4)*sin(log(x))/289 + 240*x**Rational(1, 4)*cos(log(x))/289 +
+        (16*x**Rational(1, 4)*sin(log(x))/17 + 4*x**Rational(1, 4)*cos(log(x))/17)*log(x))
 
 
 def test_issue_6828():
@@ -1223,14 +1259,6 @@ def test_issue_6828():
     g = integrate(f, x).diff(x)
     assert verify_numerically(f, g, tol=1e-12)
 
-@XFAIL
-def test_integrate_Piecewise_rational_over_reals():
-    f = Piecewise(
-        (0,                                              t - 478.515625*pi <  0),
-        (13.2075145209219*pi/(0.000871222*t + 0.995)**2, t - 478.515625*pi >= 0))
-
-    assert integrate(f, (t, 0, oo)) == 15235.9375*pi
-
 
 def test_issue_4803():
     x_max = Symbol("x_max")
@@ -1249,6 +1277,7 @@ def test_issue_4492():
         ((-2*x**5 + 15*x**3 - 25*x + 25*sqrt(-x**2 + 5)*asin(sqrt(5)*x/5)) /
             (8*sqrt(-x**2 + 5)), True))
 
+
 def test_issue_2708():
     # This test needs to use an integration function that can
     # not be evaluated in closed form.  Update as needed.
@@ -1262,6 +1291,13 @@ def test_issue_2708():
         NonElementaryIntegral(exp(-1.2*n*s*z)*exp(1.2*n*s*z**2/t),
                                   (z, 0, x))
 
+
+def test_issue_2884():
+    f = (4.000002016020*x + 4.000002016020*y + 4.000006024032)*exp(10.0*x)
+    e = integrate(f, (x, 0.1, 0.2))
+    assert str(e) == '1.86831064982608*y + 2.16387491480008'
+
+
 def test_issue_8368():
     assert integrate(exp(-s*x)*cosh(x), (x, 0, oo)) == \
         Piecewise(
@@ -1271,8 +1307,8 @@ def test_issue_8368():
                     (   1/(pi*s*(1 - 1/s**2)),
                         Abs(s**(-2)) < 1),
                     (   meijerg(
-                            ((S(1)/2,), (0, 0)),
-                            ((0, S(1)/2), (0,)),
+                            ((S.Half,), (0, 0)),
+                            ((0, S.Half), (0,)),
                             polar_lift(s)**2),
                         True)
                 ),
@@ -1326,6 +1362,12 @@ def test_issue_11856():
     t = symbols('t')
     assert integrate(sinc(pi*t), t) == Si(pi*t)/pi
 
+
+@slow
+def test_issue_11876():
+    assert integrate(sqrt(log(1/x)), (x, 0, 1)) == sqrt(pi)/2
+
+
 def test_issue_4950():
     assert integrate((-60*exp(x) - 19.2*exp(4*x))*exp(4*x), x) ==\
         -2.4*exp(8*x) - 12.0*exp(5*x)
@@ -1335,12 +1377,12 @@ def test_issue_4968():
     assert integrate(sin(log(x**2))) == x*sin(2*log(x))/5 - 2*x*cos(2*log(x))/5
 
 def test_singularities():
-    assert integrate(1/x**2, (x, -oo, oo)) == oo
-    assert integrate(1/x**2, (x, -1, 1)) == oo
-    assert integrate(1/(x - 1)**2, (x, -2, 2)) == oo
+    assert integrate(1/x**2, (x, -oo, oo)) is oo
+    assert integrate(1/x**2, (x, -1, 1)) is oo
+    assert integrate(1/(x - 1)**2, (x, -2, 2)) is oo
 
-    assert integrate(1/x**2, (x, 1, -1)) == -oo
-    assert integrate(1/(x - 1)**2, (x, 2, -2)) == -oo
+    assert integrate(1/x**2, (x, 1, -1)) is -oo
+    assert integrate(1/(x - 1)**2, (x, 2, -2)) is -oo
 
 def test_issue_12645():
     x, y = symbols('x y', real=True)
@@ -1354,57 +1396,77 @@ def test_issue_12645():
 def test_issue_12677():
     assert integrate(sin(x) / (cos(x)**3) , (x, 0, pi/6)) == Rational(1,6)
 
+
+def test_issue_14078():
+    assert integrate((cos(3*x)-cos(x))/x, (x, 0, oo)) == -log(3)
+
+
 def test_issue_14064():
     assert integrate(1/cosh(x), (x, 0, oo)) == pi/2
 
+
 def test_issue_14027():
-    assert integrate(1/(1 + exp(x - S(1)/2)/(1 + exp(x))), x) == \
-        x - exp(S(1)/2)*log(exp(x) + exp(S(1)/2)/(1 + exp(S(1)/2)))/(exp(S(1)/2) + E)
+    assert integrate(1/(1 + exp(x - S.Half)/(1 + exp(x))), x) == \
+        x - exp(S.Half)*log(exp(x) + exp(S.Half)/(1 + exp(S.Half)))/(exp(S.Half) + E)
+
 
 def test_issue_8170():
-    assert integrate(tan(x), (x, 0, pi/2)) == S.Infinity
+    assert integrate(tan(x), (x, 0, pi/2)) is S.Infinity
+
 
 def test_issue_8440_14040():
-    assert integrate(1/x, (x, -1, 1)) == S.NaN
-    assert integrate(1/(x + 1), (x, -2, 3)) == S.NaN
+    assert integrate(1/x, (x, -1, 1)) is S.NaN
+    assert integrate(1/(x + 1), (x, -2, 3)) is S.NaN
+
 
 def test_issue_14096():
     assert integrate(1/(x + y)**2, (x, 0, 1)) == -1/(y + 1) + 1/y
     assert integrate(1/(1 + x + y + z)**2, (x, 0, 1), (y, 0, 1), (z, 0, 1)) == \
         -4*log(4) - 6*log(2) + 9*log(3)
 
+
 def test_issue_14144():
     assert Abs(integrate(1/sqrt(1 - x**3), (x, 0, 1)).n() - 1.402182) < 1e-6
     assert Abs(integrate(sqrt(1 - x**3), (x, 0, 1)).n() - 0.841309) < 1e-6
 
+
 def test_issue_14375():
     # This raised a TypeError. The antiderivative has exp_polar, which
     # may be possible to unpolarify, so the exact output is not asserted here.
     assert integrate(exp(I*x)*log(x), x).has(Ei)
 
+
 def test_issue_14437():
     f = Function('f')(x, y, z)
     assert integrate(f, (x, 0, 1), (y, 0, 2), (z, 0, 3)) == \
                 Integral(f, (x, 0, 1), (y, 0, 2), (z, 0, 3))
 
+
 def test_issue_14470():
     assert integrate(1/sqrt(exp(x) + 1), x) == \
         log(-1 + 1/sqrt(exp(x) + 1)) - log(1 + 1/sqrt(exp(x) + 1))
 
+
 def test_issue_14877():
     f = exp(1 - exp(x**2)*x + 2*x**2)*(2*x**3 + x)/(1 - exp(x**2)*x)**2
     assert integrate(f, x) == \
         -exp(2*x**2 - x*exp(x**2) + 1)/(x*exp(3*x**2) - exp(2*x**2))
 
+
 def test_issue_14782():
     f = sqrt(-x**2 + 1)*(-x**2 + x)
     assert integrate(f, [x, -1, 1]) == - pi / 8
-    assert integrate(f, [x, 0, 1]) == S(1) / 3 - pi / 16
+
+
+@slow
+def test_issue_14782_slow():
+    f = sqrt(-x**2 + 1)*(-x**2 + x)
+    assert integrate(f, [x, 0, 1]) == S.One / 3 - pi / 16
 
 
 def test_issue_12081():
-    f = x**(-S(3)/2)*exp(-x)
-    assert integrate(f, [x, 0, oo]) == oo
+    f = x**(Rational(-3, 2))*exp(-x)
+    assert integrate(f, [x, 0, oo]) is oo
 
 
 def test_issue_15285():
@@ -1468,6 +1530,7 @@ def test_issue_15640_log_substitutions():
     F = -sqrt(pi)*erfc(sqrt(log(x)))/2 - sqrt(log(x))/x
     assert integrate(f, x) == F and F.diff(x) == f
 
+
 def test_issue_15509():
     from sympy.vector import CoordSys3D
     N = CoordSys3D('N')
@@ -1476,12 +1539,81 @@ def test_issue_15509():
         (-sin(a*x_1 + b)/a + sin(a*x_2 + b)/a, (a > -oo) & (a < oo) & Ne(a, 0)), \
             (-x_1*cos(b) + x_2*cos(b), True))
 
-@slow
-def test_issue_4311():
-    x = symbols('x')
-    assert integrate(x*abs(9-x**2), x) == Integral(x*abs(9-x**2), x)
+
+def test_issue_4311_fast():
     x = symbols('x', real=True)
     assert integrate(x*abs(9-x**2), x) == Piecewise(
         (x**4/4 - 9*x**2/2, x <= -3),
-        (-x**4/4 + 9*x**2/2 - S(81)/2, x <= 3),
+        (-x**4/4 + 9*x**2/2 - Rational(81, 2), x <= 3),
         (x**4/4 - 9*x**2/2, True))
+
+
+def test_integrate_with_complex_constants():
+    K = Symbol('K', real=True, positive=True)
+    x = Symbol('x', real=True)
+    m = Symbol('m', real=True)
+    assert integrate(exp(-I*K*x**2+m*x), x) == sqrt(I)*sqrt(pi)*exp(-I*m**2
+                    /(4*K))*erfi((-2*I*K*x + m)/(2*sqrt(K)*sqrt(-I)))/(2*sqrt(K))
+    assert integrate(1/(1 + I*x**2), x) == -sqrt(I)*log(x - sqrt(I))/2 +\
+        sqrt(I)*log(x + sqrt(I))/2
+    assert integrate(exp(-I*x**2), x) == sqrt(pi)*erf(sqrt(I)*x)/(2*sqrt(I))
+
+
+def test_issue_14241():
+    x = Symbol('x')
+    n = Symbol('n', positive=True, integer=True)
+    assert integrate(n * x ** (n - 1) / (x + 1), x) == \
+           n**2*x**n*lerchphi(x*exp_polar(I*pi), 1, n)*gamma(n)/gamma(n + 1)
+
+def test_issue_13112():
+    assert integrate(sin(t)**2 / (5 - 4*cos(t)), [t, 0, 2*pi]) == pi / 4
+
+
+def test_issue_14709b():
+    h = Symbol('h', positive=True)
+    i = integrate(x*acos(1 - 2*x/h), (x, 0, h))
+    assert i == 5*h**2*pi/16
+
+
+def test_issue_8614():
+    x = Symbol('x')
+    t = Symbol('t')
+    assert integrate(exp(t)/t, (t, -oo, x)) == Ei(x)
+    assert integrate((exp(-x) - exp(-2*x))/x, (x, 0, oo)) == log(2)
+
+
+def test_issue_15494():
+    s = symbols('s', real=True, positive=True)
+
+    integrand = (exp(s/2) - 2*exp(1.6*s) + exp(s))*exp(s)
+    solution = integrate(integrand, s)
+    assert solution != S.NaN
+    # Not sure how to test this properly as it is a symbolic expression with floats
+    # assert str(solution) == '0.666666666666667*exp(1.5*s) + 0.5*exp(2.0*s) - 0.769230769230769*exp(2.6*s)'
+    # Maybe
+    assert abs(solution.subs(s, 1) - (-3.67440080236188)) <= 1e-8
+
+    integrand = (exp(s/2) - 2*exp(S(8)/5*s) + exp(s))*exp(s)
+    assert integrate(integrand, s) == -10*exp(13*s/5)/13 + 2*exp(3*s/2)/3 + exp(2*s)/2
+
+
+def test_li_integral():
+    y = Symbol('y')
+    assert Integral(li(y*x**2), x).doit() == Piecewise(
+            (x*li(x**2*y) - x*Ei(3*log(x) + 3*log(y)/2)/(sqrt(y)*sqrt(x**2)), Ne(y, 0)),
+            (0, True))
+
+
+def test_issue_17473():
+    x = Symbol('x')
+    n = Symbol('n')
+    assert integrate(sin(x**n), x) == \
+        x*x**n*gamma(S(1)/2 + 1/(2*n))*hyper((S(1)/2 + 1/(2*n),),
+                     (S(3)/2, S(3)/2 + 1/(2*n)),
+                     -x**(2*n)/4)/(2*n*gamma(S(3)/2 + 1/(2*n)))
+
+
+def test_issue_17671():
+    assert integrate(log(log(x)) / x**2, [x, 1, oo]) == -EulerGamma
+    assert integrate(log(log(x)) / x**3, [x, 1, oo]) == -log(2)/2 - EulerGamma/2
+    assert integrate(log(log(x)) / x**10, [x, 1, oo]) == -2*log(3)/9 - EulerGamma/9
