@@ -281,7 +281,7 @@ def kinetic_energy(frame, *body):
 
     if not isinstance(frame, ReferenceFrame):
         raise TypeError('Please enter a valid ReferenceFrame')
-    ke_sys = S(0)
+    ke_sys = S.Zero
     for e in body:
         if isinstance(e, (RigidBody, Particle)):
             ke_sys += e.kinetic_energy(frame)
@@ -333,7 +333,7 @@ def potential_energy(*body):
 
     """
 
-    pe_sys = S(0)
+    pe_sys = S.Zero
     for e in body:
         if isinstance(e, (RigidBody, Particle)):
             pe_sys += e.potential_energy
@@ -341,13 +341,14 @@ def potential_energy(*body):
             raise TypeError('*body must have only Particle or RigidBody')
     return pe_sys
 
+
 def gravity(acceleration, *bodies):
     """
     Returns a list of gravity forces given the acceleration
     due to gravity and any number of particles or rigidbodies.
 
-    Examples
-    ========
+    Example
+    =======
 
     >>> from sympy.physics.mechanics import ReferenceFrame, Point, Particle, outer, RigidBody
     >>> from sympy.physics.mechanics.functions import gravity
@@ -372,15 +373,63 @@ def gravity(acceleration, *bodies):
         raise TypeError("No bodies(instances of Particle or Rigidbody) were passed.")
 
     for e in bodies:
-        try:
-            point = e.masscenter
-        except AttributeError:
+        point = getattr(e, 'masscenter', None)
+        if point is None:
             point = e.point
 
         gravity_force.append((point, e.mass*acceleration))
 
     return gravity_force
 
+
+def center_of_mass(point, *bodies):
+    """
+    Returns the position vector from the given point to the center of mass
+    of the given bodies(particles or rigidbodies).
+
+    Example
+    =======
+
+    >>> from sympy import symbols, S
+    >>> from sympy.physics.vector import Point
+    >>> from sympy.physics.mechanics import Particle, ReferenceFrame, RigidBody, outer
+    >>> from sympy.physics.mechanics.functions import center_of_mass
+    >>> a = ReferenceFrame('a')
+    >>> m = symbols('m', real=True)
+    >>> p1 = Particle('p1', Point('p1_pt'), S(1))
+    >>> p2 = Particle('p2', Point('p2_pt'), S(2))
+    >>> p3 = Particle('p3', Point('p3_pt'), S(3))
+    >>> p4 = Particle('p4', Point('p4_pt'), m)
+    >>> b_f = ReferenceFrame('b_f')
+    >>> b_cm = Point('b_cm')
+    >>> mb = symbols('mb')
+    >>> b = RigidBody('b', b_cm, b_f, mb, (outer(b_f.x, b_f.x), b_cm))
+    >>> p2.point.set_pos(p1.point, a.x)
+    >>> p3.point.set_pos(p1.point, a.x + a.y)
+    >>> p4.point.set_pos(p1.point, a.y)
+    >>> b.masscenter.set_pos(p1.point, a.y + a.z)
+    >>> point_o=Point('o')
+    >>> point_o.set_pos(p1.point, center_of_mass(p1.point, p1, p2, p3, p4, b))
+    >>> expr = 5/(m + mb + 6)*a.x + (m + mb + 3)/(m + mb + 6)*a.y + mb/(m + mb + 6)*a.z
+    >>> point_o.pos_from(p1.point)
+    5/(m + mb + 6)*a.x + (m + mb + 3)/(m + mb + 6)*a.y + mb/(m + mb + 6)*a.z
+    """
+    if not bodies:
+        raise TypeError("No bodies(instances of Particle or Rigidbody) were passed.")
+
+    total_mass = 0
+    vec = Vector(0)
+    for i in bodies:
+        total_mass += i.mass
+
+        masscenter = getattr(i, 'masscenter', None)
+        if masscenter is None:
+            masscenter = i.point
+        vec += i.mass*masscenter.pos_from(point)
+
+    return vec/total_mass
+
+
 def Lagrangian(frame, *body):
     """Lagrangian of a multibody system.
 
