@@ -29,13 +29,14 @@ known_fcns_src1 = ["sin", "cos", "tan", "cot", "sec", "csc",
                    "besseli", "besselj", "besselk", "bessely",
                    "bernoulli", "beta", "euler", "exp", "factorial", "floor",
                    "fresnelc", "fresnels", "gamma", "harmonic", "log",
-                   "polylog", "sign", "zeta"]
+                   "polylog", "sign", "zeta", "legendre"]
 
 # These functions have different names ("Sympy": "Octave"), more
 # generally a mapping to (argument_conditions, octave_function).
 known_fcns_src2 = {
     "Abs": "abs",
     "arg": "angle",  # arg/angle ok in Octave but only angle in Matlab
+    "binomial": "bincoeff",
     "ceiling": "ceil",
     "chebyshevu": "chebyshevU",
     "chebyshevt": "chebyshevT",
@@ -51,6 +52,7 @@ known_fcns_src2 = {
     "loggamma": "gammaln",
     "Max": "max",
     "Min": "min",
+    "Mod": "mod",
     "polygamma": "psi",
     "re": "real",
     "RisingFactorial": "pochhammer",
@@ -207,6 +209,11 @@ class OctaveCodePrinter(CodePrinter):
             return (sign + multjoin(a, a_str) +
                     divsym + "(%s)" % multjoin(b, b_str))
 
+    def _print_Relational(self, expr):
+        lhs_code = self._print(expr.lhs)
+        rhs_code = self._print(expr.rhs)
+        op = expr.rel_op
+        return "{0} {1} {2}".format(lhs_code, op, rhs_code)
 
     def _print_Pow(self, expr):
         powsymbol = '^' if all([x.is_number for x in expr.args]) else '.^'
@@ -233,6 +240,10 @@ class OctaveCodePrinter(CodePrinter):
         return '%s^%s' % (self.parenthesize(expr.base, PREC),
                           self.parenthesize(expr.exp, PREC))
 
+    def _print_MatrixSolve(self, expr):
+        PREC = precedence(expr)
+        return "%s \\ %s" % (self.parenthesize(expr.matrix, PREC),
+                             self.parenthesize(expr.vector, PREC))
 
     def _print_Pi(self, expr):
         return 'pi'
@@ -392,6 +403,16 @@ class OctaveCodePrinter(CodePrinter):
         return "double(%s == %s)" % tuple(self.parenthesize(x, prec)
                                           for x in expr.args)
 
+    def _print_HadamardProduct(self, expr):
+        return '.*'.join([self.parenthesize(arg, precedence(expr))
+                          for arg in expr.args])
+
+    def _print_HadamardPower(self, expr):
+        PREC = precedence(expr)
+        return '.**'.join([
+            self.parenthesize(expr.base, PREC),
+            self.parenthesize(expr.exp, PREC)
+            ])
 
     def _print_Identity(self, expr):
         shape = expr.shape
