@@ -38,14 +38,14 @@ v.append(N.j - (Integral(f(b)) - C.x**2)*N.k)
 upretty_v_8 = u(
 """\
       ⎛   2   ⌠        ⎞    \n\
-N_j + ⎜C_x  - ⎮ f(b) db⎟ N_k\n\
+j_N + ⎜x_C  - ⎮ f(b) db⎟ k_N\n\
       ⎝       ⌡        ⎠    \
 """)
 pretty_v_8 = u(
     """\
-N_j + /         /       \\\n\
+j_N + /         /       \\\n\
       |   2    |        |\n\
-      |C_x  -  | f(b) db|\n\
+      |x_C  -  | f(b) db|\n\
       |        |        |\n\
       \\       /         / \
 """)
@@ -56,13 +56,13 @@ v.append((a**2 + b)*N.i + (Integral(f(b)))*N.k)
 upretty_v_11 = u(
 """\
 ⎛ 2    ⎞        ⎛⌠        ⎞    \n\
-⎝a  + b⎠ N_i  + ⎜⎮ f(b) db⎟ N_k\n\
+⎝a  + b⎠ i_N  + ⎜⎮ f(b) db⎟ k_N\n\
                 ⎝⌡        ⎠    \
 """)
 pretty_v_11 = u(
 """\
 / 2    \\ + /  /       \\\n\
-\\a  + b/ N_i| |        |\n\
+\\a  + b/ i_N| |        |\n\
            | | f(b) db|\n\
            | |        |\n\
            \\/         / \
@@ -74,23 +74,23 @@ s = 3*N.x**2*C.y
 upretty_s = u(
 """\
          2\n\
-3⋅C_y⋅N_x \
+3⋅y_C⋅x_N \
 """)
 pretty_s = u(
 """\
          2\n\
-3*C_y*N_x \
+3*y_C*x_N \
 """)
 
 # This is the pretty form for ((a**2 + b)*N.i + 3*(C.y - c)*N.k) | N.k
 upretty_d_7 = u(
 """\
 ⎛ 2    ⎞                                     \n\
-⎝a  + b⎠ (N_i|N_k)  + (3⋅C_y - 3⋅c) (N_k|N_k)\
+⎝a  + b⎠ (i_N|k_N)  + (3⋅y_C - 3⋅c) (k_N|k_N)\
 """)
 pretty_d_7 = u(
 """\
-/ 2    \\ (N_i|N_k) + (3*C_y - 3*c) (N_k|N_k)\n\
+/ 2    \\ (i_N|k_N) + (3*y_C - 3*c) (k_N|k_N)\n\
 \\a  + b/                                    \
 """)
 
@@ -114,32 +114,32 @@ def test_str_printing():
 @XFAIL
 def test_pretty_printing_ascii():
     assert pretty(v[0]) == u'0'
-    assert pretty(v[1]) == u'N_i'
-    assert pretty(v[5]) == u'(a) N_i + (-b) N_j'
+    assert pretty(v[1]) == u'i_N'
+    assert pretty(v[5]) == u'(a) i_N + (-b) j_N'
     assert pretty(v[8]) == pretty_v_8
-    assert pretty(v[2]) == u'(-1) N_i'
+    assert pretty(v[2]) == u'(-1) i_N'
     assert pretty(v[11]) == pretty_v_11
     assert pretty(s) == pretty_s
     assert pretty(d[0]) == u'(0|0)'
-    assert pretty(d[5]) == u'(a) (N_i|N_k) + (-b) (N_j|N_k)'
+    assert pretty(d[5]) == u'(a) (i_N|k_N) + (-b) (j_N|k_N)'
     assert pretty(d[7]) == pretty_d_7
-    assert pretty(d[10]) == u'(cos(a)) (C_i|N_k) + (-sin(a)) (C_j|N_k)'
+    assert pretty(d[10]) == u'(cos(a)) (i_C|k_N) + (-sin(a)) (j_C|k_N)'
 
 
-def test_pretty_print_unicode():
+def test_pretty_print_unicode_v():
     assert upretty(v[0]) == u'0'
-    assert upretty(v[1]) == u'N_i'
-    assert upretty(v[5]) == u'(a) N_i + (-b) N_j'
+    assert upretty(v[1]) == u'i_N'
+    assert upretty(v[5]) == u'(a) i_N + (-b) j_N'
     # Make sure the printing works in other objects
-    assert upretty(v[5].args) == u'((a) N_i, (-b) N_j)'
+    assert upretty(v[5].args) == u'((a) i_N, (-b) j_N)'
     assert upretty(v[8]) == upretty_v_8
-    assert upretty(v[2]) == u'(-1) N_i'
+    assert upretty(v[2]) == u'(-1) i_N'
     assert upretty(v[11]) == upretty_v_11
     assert upretty(s) == upretty_s
     assert upretty(d[0]) == u'(0|0)'
-    assert upretty(d[5]) == u'(a) (N_i|N_k) + (-b) (N_j|N_k)'
+    assert upretty(d[5]) == u'(a) (i_N|k_N) + (-b) (j_N|k_N)'
     assert upretty(d[7]) == upretty_d_7
-    assert upretty(d[10]) == u'(cos(a)) (C_i|N_k) + (-sin(a)) (C_j|N_k)'
+    assert upretty(d[10]) == u'(cos(a)) (i_C|k_N) + (-sin(a)) (j_C|k_N)'
 
 
 def test_latex_printing():
@@ -151,7 +151,7 @@ def test_latex_printing():
     assert latex(v[6]) == ('(\\mathbf{{x}_{N}} + a^{2})\\mathbf{\\hat{i}_' +
                           '{N}} + \\mathbf{\\hat{k}_{N}}')
     assert latex(v[8]) == ('\\mathbf{\\hat{j}_{N}} + (\\mathbf{{x}_' +
-                           '{C}}^{2} - \\int f{\\left (b \\right )}\\,' +
+                           '{C}}^{2} - \\int f{\\left(b \\right)}\\,' +
                            ' db)\\mathbf{\\hat{k}_{N}}')
     assert latex(s) == '3 \\mathbf{{y}_{C}} \\mathbf{{x}_{N}}^{2}'
     assert latex(d[0]) == '(\\mathbf{\\hat{0}}|\\mathbf{\\hat{0}})'
@@ -161,8 +161,8 @@ def test_latex_printing():
                            'hat{k}_{N}}) + (\\mathbf{\\hat{i}_{N}}{|' +
                            '}\\mathbf{\\hat{k}_{N}})')
     assert latex(d[11]) == ('(a^{2} + b)(\\mathbf{\\hat{i}_{N}}{|}\\' +
-                            'mathbf{\\hat{k}_{N}}) + (\\int f{\\left (' +
-                            'b \\right )}\\, db)(\\mathbf{\\hat{k}_{N}' +
+                            'mathbf{\\hat{k}_{N}}) + (\\int f{\\left(' +
+                            'b \\right)}\\, db)(\\mathbf{\\hat{k}_{N}' +
                             '}{|}\\mathbf{\\hat{k}_{N}})')
 
 
@@ -171,7 +171,7 @@ def test_custom_names():
                    variable_names=['i', 'j', 'k'])
     assert A.i.__str__() == 'A.i'
     assert A.x.__str__() == 'A.x'
-    assert A.i._pretty_form == 'A_i'
-    assert A.x._pretty_form == 'A_x'
+    assert A.i._pretty_form == 'i_A'
+    assert A.x._pretty_form == 'x_A'
     assert A.i._latex_form == r'\mathbf{{i}_{A}}'
     assert A.x._latex_form == r"\mathbf{\hat{x}_{A}}"
